@@ -1,35 +1,53 @@
 #' @include hspec.R hmoment.R
 NULL
 
-#' Compute the loglikelihood function
+#' Compute the log-likelihood function
 #'
-#' The loglikelihood of the ground process of the Hawkes model.
-#' (The estimation for jump distribution is not provided.)
+#' The log-likelihood of the ground process of the Hawkes model.
+#' (The log-likelihood for mark (jump) distribution is not provided.)
 #'
 #' @param object \code{\link{hspec-class}}. The parameter values in the object are used to compute the log-likelihood.
-#' @param inter_arrival a vector of realized inter-arrival times of events. Includes inter-arrival for events that occur in all dimensions. Start with zero.
-#' @param type a vector of realized dimensions. Distinguished by numbers, 1, 2, 3, and so on. Start with zero.
-#' @param mark a vector of realized mark (jump) sizes. Start with zero.
-#' @param N a matrix of counting processes
-#' @param Nc a matrix of cumulated counting processes
-#' @param lambda0 the initial values of lambda component. Must have the same dimensional matrix with \code{hspec}.
-#' @param N0 the initial value of N
+#' @param inter_arrival A vector of realized inter-arrival times of events which includes inter-arrival for events that occur in all dimensions. Start with zero.
+#' @param type A vector of realized dimensions distinguished by numbers, 1, 2, 3, and so on. Start with zero.
+#' @param mark A vector of realized mark (jump) sizes. Start with zero.
+#' @param N A matrix of counting processes.
+#' @param Nc A matrix of counting processes weighted by mark.
+#' @param lambda_component0 The initial values of lambda component. Must have the same dimensional matrix with \code{object}.
+#' @param N0 A matrix of initial values of N.
+#' @param ... Further arguments passed to or from other methods.
 #'
 #' @seealso \code{\link{hspec-class}}, \code{\link{hfit,hspec-method}}
 #'
+#' @docType methods
+#' @rdname logLik
+#'
 #' @export
 setMethod(
   f="logLik",
   signature(object="hspec"),
   function(object, inter_arrival, type = NULL, mark = NULL, N = NULL, Nc = NULL,
-           N0 = NULL, lambda0 = NULL){
+           N0 = NULL, lambda_component0 = NULL, ...){
+
+    additional_argument <- list(...)
+    if ("lambda0" %in% names(additional_argument)) {
+
+      warning("lambda0 is deprecated; instead use lambda_component0.")
+
+      lambda_component0 <- additional_argument[["lambda0"]]
+
+    }
+
+
+
     # parameter setting
     # after parameter setting, functions mu, alpha, beta become matrices
 
+
     plist <- setting(object)
     mu <- plist$mu
     alpha <- plist$alpha
     beta <- plist$beta
+    eta <- plist$eta
     impact <- plist$impact
     rmark <- plist$rmark
     dmark <- plist$dmark
@@ -95,24 +113,27 @@ setMethod(
       mu0 <- mu
     }
 
-    # default lambda0
-    if(length(object@type_col_map) > 0 & is.null(lambda0)){
-      stop("In this model, please provide lambda0.")
+    # default lambda_component0
+    if(length(object@type_col_map) > 0 & is.null(lambda_component0)){
+      stop("In this model, please provide lambda_component0.")
     }
-    if(is.null(lambda0)) {
-      if(!exists("this_flag_represents_binding_env_is_hfit")){
-        warning("The initial values for intensity processes are not provided. Internally determined initial values are used.\n")
+    if(is.null(lambda_component0)) {
+      if(!("showWarning" %in% names(additional_argument))){
+
+        message("The initial values for intensity processes are not provided. Internally determined initial values are used.\n")
+
       }
-      lambda0 <- get_lambda0(object, mark = mark, type = type, inter_arrival = inter_arrival,
+
+      lambda_component0 <- get_lambda0(object, mark = mark, type = type, inter_arrival = inter_arrival,
                              N = N, Nc = Nc,
                              mu = mu, alpha = alpha, beta = beta)
     }
-    rowSums_lambda0 <- rowSums(matrix(lambda0, nrow=dimens))
+    rowSums_lambda_component0 <- rowSums(matrix(lambda_component0, nrow=dimens))
 
 
-    # too complicated code
+    # little bit complicated code
     if("lambda_component" %in% c(impct_args, mu_args)){
-      lambda_component <- matrix(sapply(lambda0, c, numeric(length = size - 1)), ncol = ncol(beta) * dimens)
+      lambda_component <- matrix(sapply(lambda_component0, c, numeric(length = size - 1)), ncol = ncol(beta) * dimens)
       indxM <- matrix(rep(1:ncol(beta), dimens), byrow = TRUE, nrow = dimens)
       #colnames(lambda_component) <- paste0("lambda", indxM, t(indxM))
       colnames(lambda_component) <- as.vector(t(outer(as.vector(outer("lambda", 1:dimens, FUN = paste0)),
@@ -120,7 +141,7 @@ setMethod(
     }
 
     if("lambda" %in% c(impct_args, mu_args)){
-      lambda   <- matrix(sapply(as.vector(mu0) + rowSums_lambda0, c, numeric(length = size - 1)), ncol = dimens)
+      lambda   <- matrix(sapply(as.vector(mu0) + rowSums_lambda_component0, c, numeric(length = size - 1)), ncol = dimens)
       colnames(lambda) <- paste0("lambda", 1:dimens)
     }
 
@@ -130,7 +151,7 @@ setMethod(
     sum_mu_inter_arrival <- 0
     sum_log_dmark <- 0
 
-    current_lambda <- lambda0
+    current_lambda_component_without_mu <- lambda_component0
 
     # currently only piecewise constant mu is available, hope to be updated
     if (is.function(mu)){
@@ -146,33 +167,40 @@ setMethod(
     is_lambda_necessary <- "lambda" %in% c(impct_args, mu_args)
     is_lambda_component_necessary <- "lambda_component" %in% c(impct_args, mu_args)
 
+    zero_mat_for_alpha <- matrix(rep(0, dimens * ncol(beta)), nrow = dimens)  # for fast computation, pre-initialize
+    zero_mat_for_eta <- matrix(rep(0, dimens * ncol(beta)), nrow = dimens)  # for fast computation, pre-initialize
+    zero_mat_for_impact <- matrix(rep(0, dimens * ncol(beta)), nrow = dimens)  # for fast computation, pre-initialize
+
+
     for (n in 2:size) {
 
       mu_n <- rmu_n
+      type_n <- type[n]
+      inter_arrival_n <- inter_arrival[n]
 
       # lambda decayed due to time, impact due to mark is not added yet
-      decayed <- exp(-beta * inter_arrival[n])
-      decayed_lambda <- lambda_component_n <- current_lambda * decayed
+      decayed <- exp(-beta * inter_arrival_n)
+      decayed_lambda <- lambda_component_n <- current_lambda_component_without_mu * decayed
 
       ## 1. sum of integrated_lambda_component
       sum_integrated_lambda_component <- sum_integrated_lambda_component +
-        sum(current_lambda / beta * ( 1 - decayed ))
+        sum(current_lambda_component_without_mu / beta * ( 1 - decayed ))
 
 
       ## 2. sum of log lambda when jump occurs
       if (dimens == 1) lambda_lc <- mu_n + sum(decayed_lambda)
-      else lambda_lc_type_n <- mu_n[type[n]] + sum(decayed_lambda[type[n], ])
+      else lambda_lc_type_n <- mu_n[type_n] + sum(decayed_lambda[type_n, ])
 
 
       #log(lambda_lc[type[n]]) can be NaN, so warning is turned off for a moment
-      oldw <- getOption("warn")
+      #oldw <- getOption("warn")
 
-      options(warn = -1)
-      if (dimens == 1) sum_log_lambda <- sum_log_lambda + log(lambda_lc)
-      else sum_log_lambda <- sum_log_lambda + log(lambda_lc_type_n)
+      #options(warn = -1)
+      if (dimens == 1) sum_log_lambda <- sum_log_lambda + suppressWarnings(log(lambda_lc))
+      else sum_log_lambda <- sum_log_lambda + suppressWarnings(log(lambda_lc_type_n))
 
 
-      options(warn = oldw)
+      #options(warn = oldw)
 
       ## 2.1. sum of log mark p.d.f. when jump occurs
       if(!is.null(dmark)){
@@ -187,7 +215,7 @@ setMethod(
 
 
       ## 3. sum of mu * inter_arrival
-      sum_mu_inter_arrival <- sum_mu_inter_arrival + sum(mu_n) * inter_arrival[n]
+      sum_mu_inter_arrival <- sum_mu_inter_arrival + sum(mu_n) * inter_arrival_n
 
       if(is_lambda_component_necessary)
         lambda_component[n, ] <- t(decayed_lambda)
@@ -195,25 +223,38 @@ setMethod(
         lambda[n, ] <- as.vector(mu_n) + rowSums(decayed_lambda)
 
       # impact
-      # 1. impact by alpha
 
       if( is.null(object@type_col_map) ){
-        types <- type[n]
+        types <- type_n
       } else if ( length(object@type_col_map) > 0 ) {
-        types <- object@type_col_map[[type[n]]]
+        types <- object@type_col_map[[type_n]]
       } else {
         stop("Check the type_col_map argument.")
       }
 
-      impact_alpha <- matrix(rep(0, dimens * ncol(beta)), nrow = dimens)
+      # 1. impact by alpha
+
+      impact_alpha <- zero_mat_for_alpha   # matrix
       impact_alpha[ , types] <- alpha[ , types]
 
+      current_lambda_component_without_mu <- decayed_lambda + impact_alpha
 
-      # 2. impact by mark
+      # 2. additional impact by eta
+
+      if(!is.null(eta)) {
+
+        impact_eta <- zero_mat_for_eta   # matrix
+        impact_eta[ , types] <- eta[ , types] * (mark[n] - 1)
+
+        current_lambda_component_without_mu <- current_lambda_component_without_mu + impact_eta
+      }
+
+
+      # 3. impact by impact function
       # new_lambda = [[lambda11, lambda12, ...], [lambda21, lambda22, ...], ...]
       if(!is.null(impact)){
 
-        impact_mark <- matrix(rep(0, dimens * ncol(beta)), nrow = dimens)
+        impact_mark <- zero_mat_for_impact
         impact_res <- impact(n = n, mark = mark, type = type, inter_arrival = inter_arrival,
                              N = N, Nc = Nc, lambda = lambda, lambda_component = lambda_component,
                              lambda_component_n = lambda_component_n,
@@ -222,17 +263,11 @@ setMethod(
         impact_mark[ , types] <- impact_res[ , types]
 
         # for next step
-        current_lambda <- decayed_lambda + impact_alpha + impact_mark
-
-      } else {
-
-        # for next step
-        current_lambda <- decayed_lambda + impact_alpha
+        current_lambda_component_without_mu <- current_lambda_component_without_mu + impact_mark
 
       }
 
 
-
       # new mu, i.e., right continuous version of mu
       if (is.function(mu)){
         # mu is represented by a function
